//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Borads 1.1.1
// Board : Seeed nRF52 Borads / Seeed XIAO nRF52840
// \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src
//----------------------------------------------------------------------------------------------

// 2023/11/05
// Pelipheral sketch : nRF52_XIAO_RSSI_peripheral.ino

#include <Arduino.h>
#include <Adafruit_TinyUSB.h>   // For Serial.print()
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include "Fonts/tahoma8pt7b.h"
#include "Fonts/FreeSans9pt7b.h"
#include <bluefruit.h>  // \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src

#define dataNum 8                 // mS data 4 byte, uS data 4 byte (2 data 8 byte)
union unionData {                 // Union for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;
uint32_t mS, uS;
int ss_1;
int aveRSSI[65];
bool notifyFlag = false;
bool connectedFlag = false;

// Instances of the class
Adafruit_SSD1306 display(128, 64, &Wire, -1);

// Custum Service and Characteristic
// 52c40000-8682-4dd1-be0c-40588193b485 for example
#define secondService_UUID(val) (const uint8_t[]) { \
    0x85, 0xB4, 0x93, 0x81, 0x58, 0x40, 0x0C, 0xBE, \
    0xD1, 0x4D, (uint8_t)(val & 0xff), (uint8_t)(val >> 8), 0x00, 0x00, 0xC4, 0x53 }  // ******** 0x52 --> 0x53

BLEClientService        secondService        (secondService_UUID(0x0000));
BLEClientCharacteristic dataCharacteristic   (secondService_UUID(0x0030));

void setup()
{
  // Serial port initialization
  Serial.begin(115200);
//  while (!Serial);
  delay(2000);

  // LED intialization
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);

  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);

  // Initialization of display SSD1306
  Wire.begin();
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))
  {
    Serial.println("Initialization failure of SSD1306");
  }
  display.clearDisplay();
  display.setFont(&FreeSans9pt7b);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 15);
  display.println("RSSI TEST");
  display.setCursor(0, 31);
  display.println("BLE Central");
  display.display();
  delay(5000);
  display.clearDisplay();

  // Initialization of Bruefruit
  Bluefruit.configCentralBandwidth(BANDWIDTH_MAX);  
  Bluefruit.begin(1, 1);
  Bluefruit.setName("XIAO BLE Central");

  // Custom Service Settings
  secondService.begin();
  Bluefruit.setTxPower(8);
  dataCharacteristic.setNotifyCallback(data_notify_callback);
  dataCharacteristic.begin();

  // Blue LED blinking interval setting
  Bluefruit.setConnLedInterval(100);

  // Callbacks
  Bluefruit.Central.setDisconnectCallback(disconnect_callback);
  Bluefruit.Central.setConnectCallback(connect_callback);

  // Scanner settings
  Bluefruit.Scanner.setRxCallback(scan_callback);
  Bluefruit.Scanner.restartOnDisconnect(true);
  Bluefruit.Scanner.filterRssi(-99);
  Bluefruit.Scanner.setIntervalMS(50, 50);     // interval:50mS  window:50mS  
  Bluefruit.Scanner.filterUuid(secondService.uuid);
  Bluefruit.Scanner.useActiveScan(false);
  Bluefruit.Scanner.start(0);                   // Continuous Scanning

  Serial.println("End of initialization");   
}

//******************************************************************************************
void loop()
{
  if(notifyFlag == true && connectedFlag == true)
  {   
    digitalWrite(LED_GREEN, LOW);

    // Get the reference to current connected connection
    uint16_t conn_hdl_1 = dataCharacteristic.connHandle();

    // get the RSSI value of this connection
    // monitorRssi() must be called previously (in connect callback)
    BLEConnection* connection_1 = Bluefruit.Connection(conn_hdl_1);
    ss_1 = movingAveI(connection_1->getRssi(), aveRSSI, 9);

    // Restore and display incoming data     
    mS = ud.dataBuff32[0];
    uS = ud.dataBuff32[1];

    notifyFlag = false;

    Serial.print(mS);
    Serial.print(" ");
    Serial.print(uS);
    Serial.print(" ");
    Serial.println(ss_1);  

    display.clearDisplay();
    display.setCursor(0, 15);
    display.print("mS "); display.print(mS);
    display.setCursor(0, 31);
    display.print("uS "); display.print(uS);
    display.setCursor(0, 63);
    display.print("RSSI "); display.print(ss_1);  
    display.display();
  }
  digitalWrite(LED_GREEN, HIGH);
//  delay(100);
}

//*****************************************************************************************************
// Callback invoked when scanner pick up an advertising data
void scan_callback(ble_gap_evt_adv_report_t* report)
{
  Bluefruit.Central.connect(report);
}

//****************************************************************************************************:
// Callback invoked when an connection is established
void connect_callback(uint16_t conn_handle)
{
  //RSSI monitor start
  BLEConnection* conn = Bluefruit.Connection(conn_handle);
  conn->monitorRssi();
  
//  conn = conn_handle;
  connectedFlag = false;
  if ( !secondService.discover(conn_handle) )
  {
    Serial.println("【connect_callback】 Service not find disconnect!");
    Bluefruit.disconnect(conn_handle);
    return;
  }
  Serial.println("【connect_callback】 Find Service");
  
  if ( !dataCharacteristic.discover() )
  {
    Serial.println("【connect_callback】 Characteristic not find disconnect!");
    Bluefruit.disconnect(conn_handle);
    return;
  }
  Serial.println("【connect_callback】 Find Characteristic");
  
  if ( !dataCharacteristic.enableNotify() )
  {
    Serial.println("【connect_callback】 Couldn't enable indicate for data");
    Bluefruit.disconnect(conn_handle);
    return;
  }
  Serial.println("【connect_callback】 enable indicate");
  connectedFlag = true;
}

//**********************************************************************************************************
// Callback invoked when an connection is broken
void disconnect_callback(uint16_t conn_handle, uint8_t reason)
{
  (void) conn_handle;
  (void) reason;

  Serial.print("【disconnect_callback】 Disconnected, reason = 0x");
  Serial.println(reason, HEX);
  connectedFlag = false;
}

//*********************************************************************************************************
// Callback invoked when a notifyication is received
void data_notify_callback(BLEClientCharacteristic* chr, uint8_t* data, uint16_t len)
{
  digitalWrite(LED_RED, LOW);
//  Serial.print("【notify_callback】 ");
//  Serial.println(len, HEX);
//  while(notifyFlag == true);
  memcpy(&ud.dataBuff8, data, len);
  notifyFlag = true;
  delay(1);                     // for LED
  digitalWrite(LED_RED, HIGH);
}

//************************************************************************************************ movingAveI()
// moving average(int)　n=interval+1 sum=buff[0]
int movingAveI(int indata, int* buff, int n) {
  for(int i = n; i >=2; i--) {
    buff[i] = buff[i-1];
  }
  buff[1] = indata;
  buff[0] = 0;
  for(int i = 1; i <= n; i++) {
    buff[0] = buff[0] + buff[i];
  }
  int ave = buff[0] / n;
  return(ave); 
}
