//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Borads 1.1.1
// Board : Seeed nRF52 Borads / Seeed XIAO nRF52840 Sense
// \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src
//----------------------------------------------------------------------------------------------

// 2023/11/05
// Central sketch : nRF52_XIAO_RSSI_central.ino

#include <Arduino.h>
#include <Adafruit_TinyUSB.h>   // For Serial.print()
#include <bluefruit.h>          // \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src
#include <Adafruit_SPIFlash.h>  // Need to be deleted /Documents/Arduino/libraries/SdFat

#define dataNum 8               // mS data 4 byte, uS data 4 byte (2 data 8 byte)
union unionData {               // Union for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;
uint32_t mS, uS;
bool subscribedFlag = false;    // Subscribed flag
bool connectedFlag = false;     // Connected flag
//int16_t conn;                   // Connect handler

// Custum Service and Characteristic
// 52c40000-8682-4dd1-be0c-40588193b485 for example
#define secondService_UUID(val) (const uint8_t[]) { \
    0x85, 0xB4, 0x93, 0x81, 0x58, 0x40, 0x0C, 0xBE, \
    0xD1, 0x4D, (uint8_t)(val & 0xff), (uint8_t)(val >> 8), 0x00, 0x00, 0xC4, 0x53 }  // ******** 0x52 --> 0x53

BLEService        secondService        (secondService_UUID(0x0000));
BLECharacteristic dataCharacteristic   (secondService_UUID(0x0030));

void setup()
{
  // Serial port initialization
  Serial.begin(115200);
//  while (!Serial);
  delay(2000);
  
  // LED initialization   
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT); 
   
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);


  // Initialization of Bruefruit class
  Bluefruit.configPrphBandwidth(BANDWIDTH_MAX);
  Bluefruit.configUuid128Count(15);

  Bluefruit.begin();
  Bluefruit.setTxPower(8);        // Check bluefruit.h:151 for supported values
//  Bluefruit.autoConnLed(false); // turn off LED to save power
  Bluefruit.setConnLedInterval(50);
  Bluefruit.Periph.setConnectCallback(connect_callback);
  Bluefruit.Periph.setDisconnectCallback(disconnect_callback);

  // Custom Service Settings
  secondService.begin();

  dataCharacteristic.setProperties(CHR_PROPS_NOTIFY);
  dataCharacteristic.setPermission(SECMODE_OPEN, SECMODE_NO_ACCESS);
  dataCharacteristic.setFixedLen(dataNum);
  dataCharacteristic.begin();

  // Advertisement Settings
  Bluefruit.Advertising.addFlags(BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE);
  Bluefruit.Advertising.addTxPower();
  Bluefruit.Advertising.addService(secondService);
  Bluefruit.ScanResponse.addName();
  
  Bluefruit.Advertising.restartOnDisconnect(true);
  Bluefruit.Advertising.setIntervalMS(20, 153);     // fast mode 20mS, slow mode 153mS
  Bluefruit.Advertising.setFastTimeout(30);         // fast mode 30 sec
  Bluefruit.Advertising.start(0);                   // 0 = Don't stop advertising after n seconds

  Serial.println("End of initialization");
}

//*************************************************************************************************************
void loop()
{
  {       
//    Serial.println("Connecting to Central ....");
    while(!Bluefruit.connected()) {
    }
//    Serial.println("Connected to Central");  
    
    if (Bluefruit.connected())      // If connected to the central
    {
      digitalWrite(LED_GREEN, LOW);

      mS = millis();
      uS = micros();
      ud.dataBuff32[0] = mS;
      ud.dataBuff32[1] = uS;

//      Serial.print("【LOOP】 ");
      Serial.print(mS);
      Serial.print(" ");
      Serial.println(uS);

      dataCharacteristic.notify(ud.dataBuff8, dataNum);   // Data transfer

      digitalWrite(LED_GREEN, HIGH);
    }    
    connectedFlag = false;        
  }
//  delay(100);   
}

//**************************************************************************************************************
// callback function
void connect_callback(uint16_t conn_handle)
{
//  conn = conn_handle;
  // Get the reference to current connection
  BLEConnection* connection = Bluefruit.Connection(conn_handle);

 // request to chamge parameters
  connection->requestPHY();                     // PHY 2MHz (2Mbit/sec moduration) 1 --> 2
  delay(1000);                                  // delay a bit for request to complete
  connection->requestConnectionParameter(6);    // connection interval (*1.25mS)   
  delay(1000);                                  // delay a bit for request to complete  
  Serial.println();
  Serial.print("PHY ----------> "); Serial.println(connection->getPHY());
  Serial.print("Data length --> "); Serial.println(connection->getDataLength());
  Serial.print("MTU ----------> "); Serial.println(connection->getMtu());
  Serial.print("Interval -----> "); Serial.println(connection->getConnectionInterval());
        
  char central_name[32] = { 0 };
  connection->getPeerName(central_name, sizeof(central_name));

  Serial.print("【connect_callback】 Connected to ");
  Serial.println(central_name);

  connectedFlag = true;
}

void disconnect_callback(uint16_t conn_handle, uint8_t reason)
{
  (void) conn_handle;
  (void) reason;

  Serial.print("【disconnect_callback】 Disconnected, reason = 0x");
  Serial.println(reason, HEX);

  connectedFlag = false;
}
